#*****************************************************************************
#
# Copyright (c) 2002-2011 by DEVGURU Co., LTD. All Right Reserved.
#
# NOTICE: Any modification of this file to change is not permitted 
#         unless authorized in writing by DEVGURU.
#
#*****************************************************************************/

#!/bin/bash
ROOT_UID=0 #Root Permission

#error code value
E_NOTSUPPORT=64
E_NOTROOT=67 #if not root
E_ALREADY_LOADED_DRIVER=110
E_NOT_EXIST_FILE=111
E_UNLOAD_FAILED=112
E_LOAD_FAILED=113
E_DRIVER_NOT_INSTALLED=114
E_DEVICE_CONNECTED=115
#kextfile list

DRIVER_NAME="ADS1Drv.kext"

bRet=0 #return value

bSuccess=0
# Check OS Version
OS_VERSION=`/usr/bin/defaults read "$3/System/Library/CoreServices/SystemVersion" ProductVersion`

OS_MAJOR_VERSION=`echo $OS_VERSION | cut -f2 -d'.' `
OS_MINOR_VERSION=`echo $OS_VERSION | cut -f3 -d'.' `

if [ $OS_MAJOR_VERSION -lt 4 ]
then
	echo "This OS Version is not support."
	exit $E_NOTSUPPORT
#else
	#echo "This OS Version is support."
fi

if [ "$UID" -ne "$ROOT_UID" ]
then
	echo "You need super user permission"
	exit $E_NOTROOT
fi

checkloaddrv()
{
	bRet=0
	kextstat | grep com.devguru.driver.ADS1
	find1=${?}
	kextstat | grep com.m7.driver 
	find2=${?}

	echo "${find1}, ${find2}"

	if [ "${find1}" = "0" -o "${find2}" = "0" ]; then
		echo "already loaded kext file. you unload first"
		bRet=1
	else
		echo "not find loaded kext in mm!"
		bRet=0
	fi
}

checkConnectUSBDevice()
{
	bRet=0
	a=`ls /dev | grep -c tty.ADS1*`

	if [ $a -gt 0 ]
	then
		echo "Before install driver, Eject M7 device"
		bRet=1
	else
		echo "not connected M7 device"
		bRet=0
	fi
}

loadDriver()
{
	bRet=0
	DRVNAME="${1}"
		
	echo "====== load ${DRVNAME} Driver ======="
	
	if [ -d ${DRVNAME} ]; then
		#cp -pR ./${DRVNAME} /System/Library/Extensions/
		chown -R root:wheel /System/Library/Extensions/${DRVNAME}
		chmod -R 755 /System/Library/Extensions/${DRVNAME}
		
		kextload -v /System/Library/Extensions/${DRVNAME}
	else
		echo "not exist kext files!"
		bRet=$E_NOT_EXIST_FILE
	fi
	
	if [ "${bRet}" = "0" ]; then
		echo "Success load ${1} Driver!!"
	else
		echo "fail load ${1} driver!!"
	fi
	
	echo "========= ${bRet} =========="
}

UnloadDriver()
{
	bRet=0
	DRVNAME="${1}"
	echo "====== Unload ${DRVNAME} Driver ======="
	if [ -d ${DRVNAME} ]; then
		kextunload ${DRVNAME}
		if [ "${?}" = "0" ]; then
			#rm -rf ${DRVNAME}
			bRet=${?}
		else
			echo "fail to unload kext file. you must check system logfile!"
			bRet=$E_UNLOAD_FAILED
		fi
	else
		echo "not exist kext files!"
		bRet=$E_NOT_EXIST_FILE
	fi
	
	if [ "${bRet}" = "0" ]; then
		echo "Success unload ${1} Driver!!"
	else
		echo "fail unlaod ${1} driver!!"
	fi
	
	echo "========= ${bRet} =========="
}

#START!!!!!!!!!!

if [ "${1}" = "-chkinst" ]; then
	checkloaddrv
	if [ $bRet -ne 0 ]; then
		echo "installed driver."
		exit $E_ALREADY_LOADED_DRIVER
	else
		echo "not install driver."
	fi
	
	exit $bSuccess
	
elif [ "${1}" = "-uninst" ]; then
#uninstall start
	echo "################### uninstall driver ###################"
	
	checkloaddrv
	if [ $bRet -eq 0 ]; then
		echo "not installed driver."
		exit $E_DRIVER_NOT_INSTALLED
	fi

	checkConnectUSBDevice
	
	if [ $bRet -ne 0 ]; then
		echo "unplug mobile device first"
		exit $E_DEVICE_CONNECTED
	fi
	
	if [ -d /System/Library/Extensions/${DRIVER_NAME} ]; then
		find "/System/Library/Extensions/${DRIVER_NAME}" -name "*.kext" | while read -r f;
		do
			UnloadDriver ${f}
			if [ "${bRet}" -ne "0" ]; then
				echo "${E_UNLOAD_FAILED}" > "./bRet.t"
				
			fi
		done
		
		if [ -f "./bRet.t" ]; then
			read bSuccess < "./bRet.t"
			rm "./bRet.t"
		fi
		
	else
		echo "no exist kext file"
	fi

	if [ "${bSuccess}" -eq "0" ]; then
		echo "################### SUCCESS ################### "
		rm -rf /System/Library/Extensions/${DRIVER_NAME}
	else
		echo "################### Fail to unload drivers ${bSuccess} ################### "
	fi

	exit $bSuccess

elif [ "${1}" = "-inst" ]; then
#INSTALL START
	echo "################### Install driver ###################"
	#check load drivers
	checkloaddrv

	if [ $bRet -ne 0 ]; then
		echo "already installed driver."
		echo "you want to uninstall driver. run \"preinstall.sh\""
		
		bSuccess=${E_ALREADY_LOADED_DRIVER}
	else

		cp -pR ./${DRIVER_NAME} /System/Library/Extensions/
		if [ "${?}" = "0" ]; then
			bSuccess=0	
		else
				exit $E_NOT_EXIST_FILE
		fi

		# load kext
		find . -name "*.kext" | while read -r f;
		do
			loadDriver ${f}
			if [ "${bRet}" -ne "0" ]; then
				echo "${E_LOAD_FAILED}" > "./bRet.t"
			fi
		done

		if [ -f "./bRet.t" ]; then
			read bSuccess < "./bRet.t"
			rm "./bRet.t"
		fi
		
	fi

	if [ "${bSuccess}" = "0" ]; then
		echo "################### SUCCESS ###################"
	else
		echo "################### Fail to load drivers ${bSuccess} ###################"
	fi

	exit $bSuccess

elif [ "${1}" = "-clear" ]; then
#uninstall start
	echo "################### clear kext files ###################"
	
	checkConnectUSBDevice
	
	if [ $bRet -ne 0 ]; then
		echo "unplug mobile device first"
		exit $E_DEVICE_CONNECTED
	fi

	if [ -d /System/Library/Extensions/${DRIVER_NAME} ]; then
		find "/System/Library/Extensions/${DRIVER_NAME}" -name "*.kext" | while read -r f;
		do
			echo "${f}"
			rm -rf ${f}
		done
	else
		echo "no exist kext file"
	fi
	
	rm -rf /System/Library/Extensions/${DRIVER_NAME}
	
	read -n1 -p "do you want to reboot system(Y or N)?" select 
	echo ""
	if [ "${select}" = "Y" -o "${select}" = "y" ]; then
		echo "rebooting your system now!"
		reboot
	else
		echo "You must reboot system!!!!!"
	fi
	
else
	echo "usage: ${0} -inst"
	echo "       ${0} -uninst"
	echo "       ${0} -chkinst"
	echo "       ${0} -clear"
	echo "                        "
	echo "       [-inst]: install kext driver files on system"
	echo "       [-uninst]: uninstall kext driver on system"
	echo "       [-chkint]: check driver to be installed"
	echo "       [-clear]: remove kext driver file in System/Library/Extensions folder."
fi
